package info.aspirecn.iov.sjjh.supplier.common.action;




import javax.servlet.http.HttpServletRequest;

import info.aspirecn.iov.sjjh.supplier.common.service.SupplierCommonService;
import info.aspirecn.iov.sjjh.supplier.common.util.Constants;
import info.aspirecn.iov.sjjh.supplier.common.util.SupplierProperties;
import info.aspirecn.iov.sjjh.supplier.common.vo.ChannelTypeHandleResponseObject;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import info.aspirecn.rdc.aspirecloud.node.except.utils.ErrorUtils;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;

/**
 * 
 * @author bzh
 *
 */
@Slf4j
@RestController
public class SupplierCommonAction {
	/**日志参数中的错误参数名*/
	private static final String LOG_ERROR_PARA = "respnoseCode";
	/**日志参数中的是否收费*/
	private static final String LOG_FEE_PARA = "isCharge";
	@Autowired
    private SupplierProperties supplierProperties;
	@Autowired
	private SupplierCommonService service;
	
	@ApiOperation(value = "通用通道接口", notes = "")
	@PostMapping(value = "/common/{url}")
    public ChannelTypeHandleResponseObject commonRoute(
    		@ApiParam(value="通道ID")@RequestHeader(name="channelId")  String channelId,
    		@ApiParam(value="超时时间,单位：毫秒",example = "10000")@RequestParam(name = "outTime", required = true) int outTime,
    		@ApiParam(value="请求参数JSON串")@RequestParam(name = "customBody", required = true) String customBody) {		
		HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
				.getRequest();
		request.setAttribute(Constants.LOG_UPSTREAM_RESPONSE_CODE,  Constants.LOG_UPSTREAM_DEFAULT_RESPONSE_CODE);
		String urI = request.getRequestURI();
		int index = urI.lastIndexOf("/");
		String urIStr = urI.substring(index+1);

        String productId = supplierProperties.getUrlMap().get(urIStr);
      /*  String  applicationName = supplierProperties.getApplicationName().get(urIStr);
        request.setAttribute("applicationName",applicationName);*/
		ChannelTypeHandleResponseObject ret = new ChannelTypeHandleResponseObject();
		ret.setIsCharge(Constants.IS_NOT_CHARGE);
		ret.setCode(Constants.FAIL);
        if(!StringUtils.isEmpty(productId)){
			String token =service.getToken();

			if(!"".equals(token)){
				//调用service
				ret = service.route(request, customBody, outTime,token,productId);
			}
		}else{
        	log.error("没有url对应的产品信息,urI:{},urlMap:{}",urI,supplierProperties.getUrlMap());
		}

		//把接口参数、调用结果和是否收费放入访问日志中
		if(ret.getCode() != Constants.SUCCESS) {
			request.setAttribute(LOG_ERROR_PARA, Constants.CHANNEL_LOG_ERROR_CODE);
		} else {
			request.setAttribute(LOG_ERROR_PARA, Constants.CHANNEL_LOG_SUCCESS_CODE);
		}
		request.setAttribute(LOG_FEE_PARA, ret.getIsCharge());

		try {
			request.setAttribute(Constants.CHANNEL_LOG_QUERY,
					Base64.encodeBase64String(customBody.getBytes("utf-8")));

		} catch (Exception ex) {
			ErrorUtils.captureException(ex);
			log.error("SupplierCommonAction.commonRoute.Exception{}", ex);
		}

		request.setAttribute(Constants.CHANNEL_TYPE_KEY,Constants.CHANNEL_TYPE_SYNC);
		return ret;
	}

	@ApiOperation(value = "删除token", notes = "")
	@PostMapping(value = "/deleteToken")
	public Boolean deleteToken(@RequestParam(name = "appId")String appId,@RequestParam(name = "appSecret") String appSecret){
		return service.deleteToken(appId,appSecret);
	}
}
